@@ -9,7 +9,8 @@
 
 // clazy:excludeall=qstring-allocations
 
-#include <QtTest>
+#include <QSignalSpy>
+#include <QTest>
 
 #include "../core/annotations.h"
 #include "../core/document_p.h"
@@ -31,13 +32,16 @@
 #include <QApplication>
 #include <QClipboard>
 #include <QDesktopServices>
+#include <QLineEdit>
 #include <QMenu>
 #include <QMessageBox>
 #include <QPushButton>
 #include <QScrollBar>
 #include <QTabletEvent>
 #include <QTemporaryDir>
+#include <QTemporaryFile>
 #include <QTextEdit>
+#include <QTimer>
 #include <QToolBar>
 #include <QTreeView>
 #include <QUrl>
@@ -103,6 +107,7 @@ private Q_SLOTS:
     void testFullScreenRequest();
     void testZoomInFacingPages();
     void testLinkWithCrop();
+    void testFieldFormatting();
 
 private:
     void simulateMouseSelection(double startX, double startY, double endX, double endY, QWidget *target);
@@ -215,35 +220,10 @@ void PartTest::testForwardPDF()
     const QDir workDir(QDir(tempDir.path()).filePath(dir));
     workDir.mkpath(QStringLiteral("."));
 
-    QFile f(QStringLiteral(KDESRCDIR "data/synctextest.tex"));
-    const QString texDestination = workDir.path() + QStringLiteral("/synctextest.tex");
-    QVERIFY(f.copy(texDestination));
-    QProcess process;
-    process.setWorkingDirectory(workDir.path());
-
-    const QString pdflatexPath(QStandardPaths::findExecutable(QStringLiteral("pdflatex")));
-    if (pdflatexPath.isEmpty()) {
-        QFAIL("pdflatex executable not found, but needed for the test. Try installing the respective TeXLive packages.");
-    }
-    process.start(pdflatexPath, QStringList() << QStringLiteral("-synctex=1") << QStringLiteral("-interaction=nonstopmode") << texDestination);
-    bool started = process.waitForStarted();
-    if (!started) {
-        qDebug() << "start error:" << process.error();
-        qDebug() << "start stdout:" << process.readAllStandardOutput();
-        qDebug() << "start stderr:" << process.readAllStandardError();
-    }
-    QVERIFY(started);
-
-    process.waitForFinished();
-    if (process.exitStatus() != QProcess::NormalExit || process.exitCode() != 0) {
-        qDebug() << "exit error:" << process.error() << "status" << process.exitStatus() << "code" << process.exitCode();
-        qDebug() << "exit stdout:" << process.readAllStandardOutput();
-        qDebug() << "exit stderr:" << process.readAllStandardError();
-    }
-
     const QString pdfResult = workDir.path() + QStringLiteral("/synctextest.pdf");
-
-    QVERIFY(QFile::exists(pdfResult));
+    QVERIFY(QFile::copy(QStringLiteral(KDESRCDIR "data/synctextest.pdf"), pdfResult));
+    const QString gzDestination = workDir.path() + QStringLiteral("/synctextest.synctex.gz");
+    QVERIFY(QFile::copy(QStringLiteral(KDESRCDIR "data/synctextest.synctex.gz"), gzDestination));
 
     QVERIFY(openDocument(&part, pdfResult));
     part.m_document->setViewportPage(0);
@@ -251,7 +231,8 @@ void PartTest::testForwardPDF()
     part.closeUrl();
 
     QUrl u(QUrl::fromLocalFile(pdfResult));
-    u.setFragment(QStringLiteral("src:100") + texDestination);
+    // Update this if you regenerate the synctextest.pdf somewhere else
+    u.setFragment(QStringLiteral("src:100/home/tsdgeos/devel/kde/okular/autotests/data/synctextest.tex"));
     part.openUrl(u);
     QCOMPARE(part.m_document->currentPage(), 1u);
 }
@@ -2180,6 +2161,122 @@ void PartTest::testLinkWithCrop()
     cropAction->trigger();
 }
 
+void PartTest::testFieldFormatting()
+{
+    // Test field formatting. This has to be a parttest so that we
+    // can properly test focus in / out which triggers formatting.
+    const QString testFile = QStringLiteral(KDESRCDIR "data/fieldFormat.pdf");
+    Okular::Part part(nullptr, nullptr, QVariantList());
+    part.openDocument(testFile);
+    part.widget()->resize(800, 600);
+
+    part.widget()->show();
+    QVERIFY(QTest::qWaitForWindowExposed(part.widget()));
+
+    // Field names in test document are:
+    //
+    // us_currency_fmt for formatting like "$ 1,234.56"
+    // de_currency_fmt for formatting like "1.234,56 €"
+    // de_simple_sum for calculation test and formatting like "1.234,56€"
+    // date_mm_dd_yyyy for dates like "18/06/2018"
+    // date_dd_mm_yyyy for dates like "06/18/2018"
+    // percent_fmt for percent format like "100,00%" if you enter 1
+    // time_HH_MM_fmt for times like "23:12"
+    // time_HH_MM_ss_fmt for times like "23:12:34"
+    // special_phone_number for an example of a special format selectable in Acrobat.
+    QMap<QString, Okular::FormField *> fields;
+    const Okular::Page *page = part.m_document->page(0);
+    const auto formFields = page->formFields();
+    for (Okular::FormField *ff : formFields) {
+        fields.insert(ff->name(), static_cast<Okular::FormField *>(ff));
+    }
+
+    const int width = part.m_pageView->horizontalScrollBar()->maximum() + part.m_pageView->viewport()->width();
+    const int height = part.m_pageView->verticalScrollBar()->maximum() + part.m_pageView->viewport()->height();
+
+    part.m_document->setViewportPage(0);
+
+    // wait for pixmap
+    QTRY_VERIFY(part.m_document->page(0)->hasPixmap(part.m_pageView));
+
+    part.actionCollection()->action(QStringLiteral("view_toggle_forms"))->trigger();
+
+    // Note as of version 1.5:
+    // The test document is prepared for future extensions to formatting for dates etc.
+    // Currently we only have the number format to test.
+    const auto ff_us = dynamic_cast<Okular::FormFieldText *>(fields.value(QStringLiteral("us_currency_fmt")));
+    const auto ff_de = dynamic_cast<Okular::FormFieldText *>(fields.value(QStringLiteral("de_currency_fmt")));
+    const auto ff_sum = dynamic_cast<Okular::FormFieldText *>(fields.value(QStringLiteral("de_simple_sum")));
+
+    const QPoint usPos(width * 0.25, height * 0.025);
+    const QPoint dePos(width * 0.25, height * 0.05);
+    const QPoint deSumPos(width * 0.25, height * 0.075);
+
+    const auto viewport = part.m_pageView->viewport();
+
+    QVERIFY(viewport);
+
+    auto usCurrencyWidget = dynamic_cast<QLineEdit *>(viewport->childAt(usPos));
+    auto deCurrencyWidget = dynamic_cast<QLineEdit *>(viewport->childAt(dePos));
+    auto sumCurrencyWidget = dynamic_cast<QLineEdit *>(viewport->childAt(deSumPos));
+
+    // Check that the widgets were found at the right position
+    QVERIFY(usCurrencyWidget);
+    QVERIFY(deCurrencyWidget);
+    QVERIFY(sumCurrencyWidget);
+
+    QTest::mousePress(usCurrencyWidget, Qt::LeftButton, Qt::NoModifier, QPoint(5, 5));
+    QTRY_VERIFY(usCurrencyWidget->hasFocus());
+    // locale is en_US for this test. Enter a value and check it.
+    usCurrencyWidget->setText(QStringLiteral("1234.56"));
+    // Check that the internal text matches
+    QCOMPARE(ff_us->text(), QStringLiteral("1234.56"));
+
+    // Now move the focus to trigger formatting.
+    QTest::mousePress(deCurrencyWidget, Qt::LeftButton, Qt::NoModifier, QPoint(5, 5));
+    QTRY_VERIFY(deCurrencyWidget->hasFocus());
+
+    QCOMPARE(usCurrencyWidget->text(), QStringLiteral("$ 1,234.56"));
+    QCOMPARE(ff_us->text(), QStringLiteral("1234.56"));
+
+    // And again with an invalid number
+    QTest::mousePress(usCurrencyWidget, Qt::LeftButton, Qt::NoModifier, QPoint(5, 5));
+    QTRY_VERIFY(usCurrencyWidget->hasFocus());
+
+    usCurrencyWidget->setText(QStringLiteral("131.234,567"));
+    QTest::mousePress(deCurrencyWidget, Qt::LeftButton, Qt::NoModifier, QPoint(5, 5));
+    QTRY_VERIFY(deCurrencyWidget->hasFocus());
+    // Check that the internal text still contains it.
+    QCOMPARE(ff_us->text(), QStringLiteral("131.234,567"));
+
+    // Just check that the text does not match the internal text.
+    // We don't check for a concrete value to keep NaN handling flexible
+    QVERIFY(ff_us->text() != usCurrencyWidget->text());
+
+    // Move the focus back and modify it a bit more
+    QTest::mousePress(usCurrencyWidget, Qt::LeftButton, Qt::NoModifier, QPoint(5, 5));
+    QTRY_VERIFY(usCurrencyWidget->hasFocus());
+
+    usCurrencyWidget->setText(QStringLiteral("1,234.567"));
+    QTest::mousePress(deCurrencyWidget, Qt::LeftButton, Qt::NoModifier, QPoint(5, 5));
+    QTRY_VERIFY(deCurrencyWidget->hasFocus());
+
+    QCOMPARE(usCurrencyWidget->text(), QStringLiteral("$ 1,234.57"));
+
+    // Sum should already match
+    QCOMPARE(sumCurrencyWidget->text(), QStringLiteral("1.234,57€"));
+
+    // Set a text in the de field
+    deCurrencyWidget->setText(QStringLiteral("1,123,234.567"));
+    QTest::mousePress(usCurrencyWidget, Qt::LeftButton, Qt::NoModifier, QPoint(5, 5));
+    QTRY_VERIFY(usCurrencyWidget->hasFocus());
+
+    QCOMPARE(deCurrencyWidget->text(), QStringLiteral("1.123.234,57 €"));
+    QCOMPARE(ff_de->text(), QStringLiteral("1,123,234.567"));
+    QCOMPARE(sumCurrencyWidget->text(), QStringLiteral("1.124.469,13€"));
+    QCOMPARE(ff_sum->text(), QStringLiteral("1,124,469.1340000000782310962677002"));
+}
+
 } // namespace Okular
 
 int main(int argc, char *argv[])
